/* MayaDate 1.0 for PalmOS 2.0 and greater. */

#include <PalmOS.h>
#include "callback.h"
#include "mayadateRsc.h"
#include <math.h>
#include <Vga.h>
#include <Silk.h>

#define mayadatePrefsVersionNum 1.1
#define mayadatePrefID 0x00
#define standardJDCorr 584285
#define mayadateCreator 'Maya'

#define AMOD(x,y) (1 + (((x) - 1) % (y)))


typedef struct {
    Boolean oldcorr;
    Boolean usebc;
} mayaPreferenceType;


typedef enum {
    goPrevious = 0,
    goNext,
    goFirstInBaktun
} GoWhere;

typedef struct longdate
{
    UInt16 bak,kat,tun,uin,kin;
} longdate;


void InitDateStuff(void);

// More global variables than I'd like, but oh well.
UInt32 SavedJulian = 0;
Boolean FirstFlag = true;
UInt32 JD_CORRELATION=584285;
UInt32 MAXJUL = 3464284;
UInt16 savedmonth = 1, savedday = 1;
Int16 savedyear = 1123;
Char ButtonArray[42][4];
Int16 hm1, hm2, hd1, hd2, td1, td2, tv1, tv2;
GoWhere Next = goNext;
DmOpenRef handeradb;
Boolean USEBC = false;
Boolean DATEONEDGE = false;
Boolean YEARONEDGE = false;

double trunc(double a) {
    if (a >= 0)
	return floor(a);
    else
	return ceil(a);
}

void SizeForm(FormPtr frm) {
    RectangleType rect;
    
    rect.topLeft.x = 0;
    rect.topLeft.y = 0;
    
    WinGetDisplayExtent(&rect.extent.x, &rect.extent.y);
    WinSetWindowBounds(FrmGetWindowHandle(frm), &rect);
}

void FixYear(Char* string, Int16 yy) {
    Char newstring[70];
    Char ministring[2];
    Char yearstring[10];
    Boolean bc = false;
    Int16 i, y;


    StrCopy(newstring, "");

    if (USEBC) {
	if (yy <= 0) {
	    bc = true;
	    y = -yy +1;
	} else
	    y = yy;
	StrIToA(yearstring, y);
	if (bc)
	    StrCat(yearstring, " BC");
	else
	    StrCat(yearstring, " AD");
    } else {
	StrIToA(yearstring, yy);
    }	
	
    
    for (  i = 0 ; i < StrLen(string)  ; i++) {
	if (string[i] == '1')
	    if (string[i+1] == '1')
		if (string[i+2]== '2')	
		    if (string[i+3] == '3') {
			StrCat(newstring, yearstring);
			i = i + 3;
			goto mark;
		    }
	ministring[0] = string[i];
	ministring[1] = '\0';
	StrCat(newstring, ministring);
    mark:
    }
    StrCopy(string, newstring);

}

Int16 MayaLoadPrefs(mayaPreferenceType *prefsP) {
    UInt16  prefsSize;
    Int16   prefsVersion = noPreferenceFound;
    
    ErrNonFatalDisplayIf(!prefsP, "null prefP arg");
    
    // Read the preferences / saved-state information.  Fix-up if no
    // prefs or older/newer version
    prefsSize = sizeof (mayaPreferenceType);
    prefsVersion = PrefGetAppPreferences (mayadateCreator, 
					  mayadatePrefID, 
					  prefsP, &prefsSize, true);
    
    // If the preferences version is from a future release (as can
    // happen when going back and syncing to an older version of the
    // device), treat it the same as "not found" because it could be
    // significantly different
    if ( prefsVersion > mayadatePrefsVersionNum )
	prefsVersion = noPreferenceFound;
                
    if ( prefsVersion == noPreferenceFound ) {
	prefsP->oldcorr = false; // default preference is to use new corresp.
    }
    
    return prefsVersion;
}

void MayaSavePrefs (void) {
        mayaPreferenceType prefs;
        
	prefs.oldcorr = true;
	if (JD_CORRELATION == standardJDCorr)
	    prefs.oldcorr = false;
	prefs.usebc = USEBC;
	PrefSetAppPreferences(mayadateCreator, mayadatePrefID, 
			      mayadatePrefsVersionNum, &prefs, 
			      sizeof (mayaPreferenceType), true);
}





// stuff when starting the app
static UInt16 StartApplication(void) {
    FrmGotoForm(MainForm);
    return 0;
}

// stuff when ending the app
static void StopApplication(void) {
    CALLBACK_PROLOGUE
	MayaSavePrefs();
    if (handeradb)
	DmCloseDatabase(handeradb);
    CALLBACK_EPILOGUE
}

//=======================================================================
// GetObjectPtr
//
// return a pointer to an object with objectID in the form on formP. use
// the active form if formP is NULL.
//=======================================================================

static MemPtr GetObjectPtr( FormPtr formP, UInt16 objectID)
{
        FormPtr frmP;

        if ( formP != NULL )
                frmP = formP;
        else
                frmP = FrmGetActiveForm();
                
        return FrmGetObjectPtr( frmP, FrmGetObjectIndex( frmP, objectID ) );

}       // GetObjectPtr

//=======================================================================
// SetFieldTextFromStr
//
// given a formPtr and fieldID, set the text of the field from the string
// on strP (a copy is made). the active form is used if formP is NULL. works
// even if the field is not editable.
//
// return a pointer to the field.
// 
//=======================================================================
FieldPtr SetFieldTextFromStr( FormPtr formP, UInt16 fieldID, MemPtr strP, Boolean redraw ) {
    MemHandle       oldTxtH;
    MemHandle       txtH;
    MemPtr          txtP;
    
    FieldPtr        fldP;
    Boolean         fieldWasEditable;
        
    
    // get some space in which to stash the string.
    txtH    = MemHandleNew(  StrLen( strP ) + 1 );
    txtP    = MemHandleLock( txtH );
    
    
    // copy the string.
    StrCopy( txtP, strP );

    
    // unlock the string handle.
    MemHandleUnlock( txtH );
    txtP = 0;
    

    // get the field and the field's current text handle.
    fldP            = GetObjectPtr( formP, fieldID );       
    oldTxtH = FldGetTextHandle( fldP );
    
        
    // set the field's text to the new text.
    fieldWasEditable                = fldP->attr.editable;
    fldP->attr.editable     = true;
    
    FldSetTextHandle( fldP, txtH );

    if (redraw) 
	FldDrawField( fldP );
    
    fldP->attr.editable     = fieldWasEditable;
    
    
    // free the handle AFTER we call FldSetTextHandle().
    if ( oldTxtH != NULL ) MemHandleFree( oldTxtH );
    
    return fldP;
    
}       // SetFieldTextFromStr


// The next routines are from the mtools package by Sean Dwyer.  I
// chopped 'em up a lot but he retains appropriate copyrights under
// the GPL.  

// The routines are still largely Sean's, but the sticky internals are
// from Peter Baum; something I was doing was messing up some dates,
// so I switched to use a form that I'd used earlier in some HP48
// Julian day number routines, which were from Peter.

// Get Julian from gregorian date.
UInt32 jul_from_date(UInt16 d, UInt16 m, Int16 y) {
    Int32 mm=m, yy=y;
    UInt32 jul;


    if (mm < 3) {
	mm = mm + 12;
	yy--;
    }

    jul = ceil(d + trunc((153 * mm - 457) / 5) + 365 * yy + floor(y / 4) - floor(yy / 100) + floor(yy / 400) + 1721118.5);

    return jul;
}

void date_from_jul(UInt32 j, UInt16* d, UInt16* m, Int16* y) { 
    Int32 z;
    double r, g;
    Int32 a, b, c;
    UInt16 dd, mm;
    Int32 yy;

    z = floor(j - 1721118.5);
    r = j - 1721118.5 - z; 
    g = z - .25;
    a = floor(g / 36524.25);
    b = a - floor(a / 4);

    yy = floor((b+g) / 365.25);

    c = b + z - floor(365.25 * yy);
    mm = trunc((5 * c + 456) / 153); 

    dd = c - trunc((153 * mm - 457) / 5) + r;
 
    if (mm > 12) {
	mm = mm - 12;
        yy = yy + 1;
    }

    *d = dd;
    *m = mm;
    *y = yy;

}

/* try to pick the right mod to use regardless of sign; this one is
   Sean's. */

double my_mod(double x, double y)
{
        double result;
        
        if ((x < 0) && (y < 0))
                /* both -ve */
                result = (Int32) x % (Int32) y;
        else if ((x > 0) && (y < 0))
                /* x is +ve, y is -ve */
                result = (Int32) x % (Int32) y;
        else if ((x < 0) && (y > 0))
                /* x is -ve, y is +ve */
                result = (Int32) x % (Int32) y ;
        else
                /* both +ve */
                result = (Int32) x % (Int32) y;
        
        return result;
}

/*
  These next few are   Ivan Van Lanaingham's; his   copyright follows.
  Copyright & Disclaimer


(C) 1997,1998, Copyright by Ivan Van Laningham; All Rights Reserved.
Mail to: ivanlan@callware.com

Permission to use, copy, modify, and distribute this software and its
documentation for any purpose and without fee or royalty is hereby granted,
provided that the above copyright notice appear in all copies and that both
the copyright notice and this permission notice appear in supporting
documentation or portions thereof, including modifications, that you make.

THE AUTHOR IVAN VAN LANINGHAM DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE!
*/


/* Take a julian date, and convert it to a Long Count */

void jdntolong (UInt32 jdn, longdate *thedate)
{
    double lc;
    double r;
    
    if (jdn > JD_CORRELATION) {
	lc = jdn - JD_CORRELATION;
    } else if (jdn == JD_CORRELATION) {
	lc = 0.0;
    } else {
	lc = jdn - JD_CORRELATION;
    }

    thedate->bak=floor(lc/144000.0);
    r=my_mod(lc,144000.0);
    thedate->kat=floor(r/7200);
    r=my_mod(r,7200.0);
    thedate->tun=floor(r/360);
    r=my_mod(r,360.0);
    thedate->uin=floor(r/20);
    thedate->kin=floor(my_mod(r,20.0));
}

UInt32 longtojdn (longdate *thedate)
{
    UInt32 result, r2;
    
    result = (double)((thedate->bak*144000) + 
                      (thedate->kat*7200) +
                      (thedate->tun*360) +
                      (thedate->uin*20) +
                      thedate->kin);
    
    r2 = result+JD_CORRELATION;
        
    return r2;
}


/* take a julian day number and convert it to a tzolkin_date */

const Char *get_tzolkin_date(UInt32 p, UInt16 *tzolkinnumber, UInt16 *tzolkinday)
{
    UInt16 number, name;
    UInt32 ilc;
    static const char *mnth[] =
        { "START","Imix","Ik","Akbal","Kan","Chicchan","Cimi","Manik",
          "Lamat","Muluc","Oc","Chuen","Eb","Ben","Ix","Men","Cib",
          "Caban","Eznab","Cauac","Ahau" };
        
    ilc = p - 1  - JD_CORRELATION;
    number = (ilc + 4 ) % 13 + 1;  
    name = (ilc + 20) % 20 + 1;
    
    *tzolkinnumber = number;
    *tzolkinday = name;
    return mnth[name];
}



const Char *get_haab_date(UInt32 x, UInt16 *haabday, UInt16 *haabmo)
{
        UInt32 lc=x-JD_CORRELATION;
        double doh;
        Int16 day, month;
        static const char *mnth[] = 
            { "START","Pop","Uo","Zip","Zotz","Tzec","Xul","Yaxkin","Mol",
              "Chen","Yax","Zac","Ceh","Mac","Kankin","Muan","Pax","Kayab",
              "Cumku","Uayeb" };

        doh = my_mod((lc + 8.0 + 20.0 * (18.0 -1.0)),365.0);
        day = floor( (my_mod(doh,20.0)));
        month = floor(doh/20.0) + 1;

        *haabday = day;
	*haabmo = month;
        return mnth[month];             
}

// Here endeth the inclusion.

// Format the longcount string up all nice
void MyFormatLC(longdate thedate, Char *longdate) {
    Char baktun[3], katun[3], tun[3], uinal[3], kin[3];

    StrIToA(baktun, thedate.bak);
    StrIToA(katun, thedate.kat);
    StrIToA(tun, thedate.tun);
    StrIToA(uinal, thedate.uin);
    StrIToA(kin, thedate.kin);
    
    StrCopy(longdate, baktun);
    StrCat(longdate, ".");
    StrCat(longdate, katun);
    StrCat(longdate, ".");
    StrCat(longdate, tun);
    StrCat(longdate, ".");
    StrCat(longdate, uinal);
    StrCat(longdate, ".");
    StrCat(longdate, kin);
}
    
void MyDrawGlyph(UInt16 bitmap, UInt16 x, UInt16 y) {
    MemHandle ObjectBitmapHandle;
    BitmapPtr ObjectBitmapPtr;
    
    ObjectBitmapHandle = DmGetResource( bitmapRsc, bitmap);
    ObjectBitmapPtr = MemHandleLock(ObjectBitmapHandle);

    WinDrawBitmap(ObjectBitmapPtr, x, y);

    MemPtrUnlock(ObjectBitmapPtr);
    DmReleaseResource(ObjectBitmapHandle);

}

void MyDrawData(UInt32 jul) {
    Char datestring[20], julstring[15], haab[15], haabnum[3], 
	tzolkin[20], tzolnum[3], longcount[20];
    const Char* haabmonth[10];
    const Char* tzolkinname[15];
    UInt16 haabday, haabmo, tzolkinnumber, tzolkinday, dd, mm;
    UInt32 version;
    Int16  yy;
    longdate thedate;
    DateFormatType dateFormat;
    Coord xextent, yextent;

    // Format up the Gregorian date
    dateFormat = PrefGetPreference(prefLongDateFormat);
    date_from_jul(jul, &dd, &mm, &yy);
    DateToAscii(mm, dd, 1123, dateFormat, datestring);
    FixYear(datestring, yy);

    // The Julian day number
    StrIToA(julstring, jul); 
    
    // Get the Long Count
    jdntolong(jul, &thedate);
    MyFormatLC(thedate, longcount);

    // Get the Tzolkin
    *tzolkinname = get_tzolkin_date(jul, &tzolkinnumber, &tzolkinday);
    StrIToA(tzolnum, tzolkinnumber);
    StrCopy(tzolkin, tzolnum);
    StrCat(tzolkin, " ");
    StrCat(tzolkin, *tzolkinname);
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	MyDrawGlyph((4096+100+tzolkinnumber), 4, 126);
	MyDrawGlyph((4096+200+tzolkinday), 30, 126);
    } else {
	MyDrawGlyph((100+tzolkinnumber), 3, 84);
	MyDrawGlyph((200+tzolkinday), 20, 84);
    }

    // Get the Haab
    *haabmonth = get_haab_date(jul, &haabday, &haabmo);
    StrIToA(haabnum, haabday);
    StrCopy(haab, haabnum);
    StrCat(haab, " ");
    StrCat(haab, *haabmonth);
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	MyDrawGlyph((4096+100+haabday), 124, 126);
	MyDrawGlyph((4096+300+haabmo), 150, 126);
    } else {
    MyDrawGlyph((100+haabday), 83, 84);
    MyDrawGlyph((300+haabmo), 100, 84);
    }

    //Print things in appropriate places.
    SetFieldTextFromStr(FrmGetActiveForm(), DateField, datestring, true);
    SetFieldTextFromStr(FrmGetActiveForm(), JulianField, julstring, true);
    SetFieldTextFromStr(FrmGetActiveForm(), LongCountField, longcount, true);
    SetFieldTextFromStr(FrmGetActiveForm(), TzolkinField, tzolkin, true);
    SetFieldTextFromStr(FrmGetActiveForm(), HaabField, haab, true);

    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	WinGetDisplayExtent(&xextent, &yextent);
	if (yextent > 240) {
	    MyDrawGlyph((100+thedate.bak), 5, 241);
	    MyDrawGlyph((100+thedate.kat), 53, 241);
	    MyDrawGlyph((100+thedate.tun), 101, 241);
	    MyDrawGlyph((100+thedate.uin), 149, 241);
	    MyDrawGlyph((100+thedate.kin), 197, 241);
	}
    }	    

    SavedJulian = jul;
}    

void InitDateStuff(void) {
    UInt32 seconds;
    UInt16 uday, umonth, uyear;
    DateTimeType date;
    mayaPreferenceType prefs;
    Int16 prefsVersion;
    
    seconds=TimGetSeconds();
    TimSecondsToDateTime(seconds, &date);

    umonth = date.month;
    uday = date.day;
    uyear = date.year;

    prefsVersion = MayaLoadPrefs(&prefs);
    if (prefs.oldcorr)
	JD_CORRELATION = JD_CORRELATION - 2;
    MAXJUL = JD_CORRELATION + 2879999;
    USEBC = prefs.usebc;
    SavedJulian = 0 + jul_from_date(uday, umonth, uyear);
}

void DoFirstDraw(void) {
    UInt16 m, d;
    Int16 y;
    date_from_jul(SavedJulian, &d, &m, &y);
    MyDrawData(SavedJulian);
}

void CopyToClipboard(UInt32 jul) {
    longdate thedate;
    Char outstring[70], haabnum[3], tzolnum[3], longcount[20];
    const Char* haabmonth[10];
    const Char* tzolkinname[15];
    UInt16 haabday, haabmo, tzolkinnumber, tzolkinday;
    


    jdntolong(jul, &thedate);
    MyFormatLC(thedate, longcount);
    StrCopy(outstring, longcount);
    StrCat(outstring, " ");

    // Get the Tzolkin
    *tzolkinname = get_tzolkin_date(jul, &tzolkinnumber, &tzolkinday);
    StrIToA(tzolnum, tzolkinnumber);
    StrCat(outstring, tzolnum);
    StrCat(outstring, " ");
    StrCat(outstring, *tzolkinname);

    // Get the Haab
    *haabmonth = get_haab_date(jul, &haabday, &haabmo);
    StrIToA(haabnum, haabday);
    StrCat(outstring, haabnum);
    StrCat(outstring, " ");
    StrCat(outstring, *haabmonth);

    ClipboardAddItem(clipboardText, &outstring, StrLen(outstring));

}
    

void DoPrefs(Boolean *check, Boolean *usebc) {
    FormPtr previousForm = FrmGetActiveForm();
    FormPtr frm = FrmInitForm(PrefsForm);
    UInt16 hitButton;
    UInt32 version;

    ControlPtr checkold = FrmGetObjectPtr(frm, 
					  FrmGetObjectIndex(frm, 
							    CorrelationCheck));
    ControlPtr checkbc = FrmGetObjectPtr(frm, 
					  FrmGetObjectIndex(frm, 
							    BCCheck));
    Boolean newValueOld;
    Boolean newValueBC;
    
    FrmSetActiveForm(frm);
    CtlSetValue(checkold, *check);
    CtlSetValue(checkbc, *usebc);
    

    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
    }
    hitButton = FrmDoDialog(frm);
    
    newValueOld = CtlGetValue(checkold);
    newValueBC = CtlGetValue(checkbc);

    if (previousForm)
	FrmSetActiveForm(previousForm);
    
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }

    FrmDeleteForm(frm);
    
    if (hitButton == CorrOKButton) {
	*usebc = newValueBC;
	*check = newValueOld;
    } 
    return;
}



UInt32 JulToday(void) {
    UInt32 seconds, jul;
    UInt16 uday, umonth; 
    Int16 uyear;
    DateTimeType date;
    
    seconds=TimGetSeconds();
    TimSecondsToDateTime(seconds, &date);

    umonth = date.month;
    uday = date.day;
    uyear = (Int16) date.year;
    jul = jul_from_date(uday, umonth, uyear);
    return jul;
}

// THis is not _quite_ the Gregorian leap year; it includes the
// "Divisible by 4000 is not a leap year" rule, which is a more modern
// innovation. It only impacts the future, though, so since we're
// using it now, we might as well keep using it.

Boolean IsLeap(Int16 yy) {
    Boolean leap = false;
    
    if ((yy % 4) == 0) {
	leap = true;
	if ((yy % 100) == 0) {
	    leap = false;
	    if ((yy % 400) == 0) {
		leap = true;
		if ((yy % 4000) == 0) {
		    leap = false;
		}
	    }
	}
    }
    return leap;
}


UInt16 DOWFromJul(UInt32 jul){
    Int16 dd, mm, yy, dow;
    Boolean leap = false, janfeb = false;
    date_from_jul(jul, &dd, &mm, &yy);
    if ((mm == 1) || (mm == 2))
	janfeb = true;
    
    leap = IsLeap(yy);

    if (janfeb && leap)
	dow = jul % 7;
    else
	dow = ((jul + 1) % 7);

    return dow;
}

// This is needed for January and February of leap years - the above
// displays right but the buttons clicked are wrong.  Strangeness.

UInt16 BadDOWFromJul(UInt32 jul) { 
    return ((jul +1) % 7);
}


// Palm's DaysInMonth() returns that every xx00 year is a leap year,
// which is wrong.
UInt16 MyDaysInMonth(Int16 mm, Int16 yy) {
    UInt16 days;
    switch (mm) {
    case 4:
    case 6:
    case 9:
    case 11:
	days = 30;
	break;

    case 1:
    case 3:
    case 5:
    case 7:
    case 8:
    case 10:
    case 12:
	days = 31;
	break;

    case 2:
	days = 28;
	if (IsLeap(yy))
	    days++;
	break;
    default:
	days = 0;
    }
    return days;
}
	    


void DrawMyDayForm(Boolean redraw) {
    UInt16 mm, dd;
    Int16 yy, y, i, offset;
    UInt32 jul = SavedJulian;
    Char year[7];
    Char blank[3];
    Char tempstring[3];
    FormPtr form = FrmGetActiveForm();
    Boolean bc = false;
    
    StrCopy(blank, "  ");

    date_from_jul(jul, &dd, &mm, &yy);

    if (redraw) {
	yy = savedyear;
	mm = savedmonth;
	dd = savedday; 
    }
    
    if (USEBC) {
	if (yy <= 0) {
	    bc = true;
	    y = -yy +1;
	} else
	    y = yy;
	StrIToA(year, y);
	if (bc)
	    StrCat(year, "BC");
    } else {
	StrIToA(year, yy);
    }	
    

    SetFieldTextFromStr(form, YearField, year, redraw);
    FrmSetControlGroupSelection(form, 1, (6100 + mm));

    offset = DOWFromJul(jul_from_date(1, mm, yy));
    if (!(yy % 100) && ((mm == 1) || (mm == 2)) && !(IsLeap(yy)))
	offset = offset - 1;

    if (YEARONEDGE) {
	for ( i = 1 ; i < 13 ; i++)
	    FrmShowObject(form, FrmGetObjectIndex(form, 6100+i));
	YEARONEDGE = false;
    }
	

    if (DATEONEDGE) {
	for (i = 7001 ; i < 7043 ; i++)
	    FrmShowObject(form, FrmGetObjectIndex(form, i));
	DATEONEDGE = false;
    }

    if (yy == -3113) {
	FrmHideObject(form, FrmGetObjectIndex(form, JanPush));
	FrmHideObject(form, FrmGetObjectIndex(form, FebPush));
	FrmHideObject(form, FrmGetObjectIndex(form, MarPush));
	FrmHideObject(form, FrmGetObjectIndex(form,AprPush));
	FrmHideObject(form, FrmGetObjectIndex(form, MayPush));
	FrmHideObject(form, FrmGetObjectIndex(form, JunPush));
	FrmHideObject(form, FrmGetObjectIndex(form, JulPush));
	if (mm < 8) {
	    mm = 8;
	    FrmSetControlGroupSelection(form, 1, (6100 + mm));
	}
	if (mm == 8) {
	    for (i = 7001 ; i < 7018 ; i++)
		FrmHideObject(form, FrmGetObjectIndex(form, i));
	    DATEONEDGE = true;
	}
	YEARONEDGE = true;
    }


    if (yy == 4772) {
	FrmHideObject(form, FrmGetObjectIndex(form, NovPush));
	FrmHideObject(form, FrmGetObjectIndex(form, DecPush));

	if (mm > 10) {
	    mm = 10;
	    FrmSetControlGroupSelection(form, 1, (6100 + mm));
	}
	if (mm == 10) {
	    for (i = 7015 ; i < 7043 ; i++)
		FrmHideObject(form, FrmGetObjectIndex(form, i));
	    DATEONEDGE = true;
	}
	YEARONEDGE=true;
    }



    for ( i = 7001 ; i < 7043 ; i++) {
	if ((i - offset) < 7001) {
	    StrCopy(ButtonArray[i - 7001], "  ");
	} else if ((i - offset) > (7000 + MyDaysInMonth(mm, yy))) {
	    StrCopy(ButtonArray[i - 7001], "  ");
	} else {
	    StrPrintF(tempstring, "%2d", (i - 7000 - offset));
	    StrCopy(ButtonArray[i - 7001], tempstring);
	}
	CtlSetLabel(
		    FrmGetObjectPtr(form , FrmGetObjectIndex(form , i)) , 
		    ButtonArray[i - 7001]);
	}

	
	

    savedday = dd;
    savedmonth = mm;
    savedyear = yy;

    return;
}

void MyDayButtons(EventPtr event){
    Int16 yy = savedyear;
    Boolean handled = false;

    switch (event->data.ctlSelect.controlID) {
    case CenturyBackButton:
	yy = yy - 100;
	if ( yy < -3113)
	    yy = yy + 100;
	handled = true;
	break;

    case DecadeBackButton:
	yy = yy - 10;
	if ( yy < -3113)
	    yy = yy + 10;
	handled = true;
	break;

    case YearBackButton:
	yy = yy - 1;
	if ( yy < -3113)
	    yy = yy + 1;
	handled = true;
	break;

    case YearForwardButton:
	yy = yy + 1;
	if ( yy > 13052)
	    yy = yy - 1;
	handled = true;
	break;

    case DecadeForwardButton:
	yy = yy + 10;
	if ( yy > 13052)
	    yy = yy - 10;
	handled = true;
	break;

    case CenturyForwardButton:
	yy = yy + 100;
	if ( yy > 13052)
	    yy = yy - 100;
	handled = true;
	break;
	
    }
    savedyear = yy;
    DrawMyDayForm(true);
}


static Boolean MyDayPushButtons(EventPtr event) {
    UInt16 hitButton;
    Boolean handled = false;
    
    
    hitButton = event->data.ctlSelect.controlID;

     if ((hitButton > 6100) && (hitButton < 6113)) {
	savedmonth = hitButton - 6100;
	handled = true;
	DrawMyDayForm(true);

    }
    return handled;
}

static Boolean MyDayHandleEvent(EventPtr event){
    Boolean handled = false;
    
    switch (event->eType) {
	
    case ctlRepeatEvent:
	MyDayButtons(event);
	break;

    case ctlSelectEvent:
	handled = MyDayPushButtons(event);
	break;
    }	

    return handled;
}


UInt32 MySelectDay(UInt32 jul) { 
    FormPtr previousForm = FrmGetActiveForm();
    FormPtr frm = FrmInitForm(MyDateSelForm);
    UInt16 hitButton, offset;
    UInt32 oldjul = jul;
    UInt32 newjul, tempjul;
    UInt32 version;


    SavedJulian = jul;

    FrmSetEventHandler(frm, MyDayHandleEvent);
    FrmSetActiveForm(frm);
    DrawMyDayForm(false);

    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
    }
    hitButton = FrmDoDialog(frm);

    if (previousForm)
	FrmSetActiveForm(previousForm);

    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }


    FrmDeleteForm(frm);
    
    if (hitButton == DateTodayButton) {
	newjul = JulToday();
	return newjul;
    } else if ((hitButton > 7000) && (hitButton < 7043)) {
	tempjul = jul_from_date(1, savedmonth, savedyear) ; 
	offset = BadDOWFromJul(tempjul);
	if ((savedmonth == 1) && IsLeap(savedyear))
	    offset = DOWFromJul(tempjul) +1;
	newjul = tempjul + (hitButton - 7000) - offset -1;
	return newjul;
    }else
	return oldjul;
}

    
UInt32 SelectJulianDay(UInt32 jul) { 
    FormPtr previousForm = FrmGetActiveForm();
    FormPtr frm = FrmInitForm(JulianForm);
    UInt16 hitButton;
    UInt32 newjul;
    FieldPtr field;
    MemHandle handle;
    MemPtr txtP;
    Boolean dirtyflag = false;
    Char julstring[15];
    Char newValue[15];
    Char maxString[15];
    Char minString[15];
    UInt32 version;


    CALLBACK_PROLOGUE


    StrIToA(maxString, MAXJUL);
    StrIToA(minString, JD_CORRELATION);

    handle = MemHandleNew(11);
    
    StrIToA(julstring, jul);
    
    FrmSetActiveForm(frm);
    SetFieldTextFromStr(FrmGetActiveForm(), JulianEnterField, julstring, false);
    field = (FieldPtr) GetObjectPtr(frm, JulianEnterField);
    field->attr.editable  = true;
    field->attr.numeric = true;
	
    FrmSetFocus(frm, FrmGetObjectIndex(frm, JulianEnterField));
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
    }
    
    
    hitButton = FrmDoDialog(frm);
    if (FldDirty(field)) {
	txtP = FldGetTextPtr(field);
	if (txtP != NULL) {
	    StrCopy(newValue, txtP);
	}
	dirtyflag = true;
    }
	
    FldSetTextHandle (field, NULL);
    
    if (previousForm)
	FrmSetActiveForm(previousForm);
    
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }


    FrmDeleteForm(frm);
    
    if (hitButton == JulOKButton) {
	newjul =  jul;
	if (dirtyflag)
	    newjul = StrAToI(newValue);
	if (newjul < JD_CORRELATION ) {
	    newjul = JD_CORRELATION;
	    FrmCustomAlert(LowJulianAlert, minString, "", "");
	} else if (newjul > MAXJUL) {
	    newjul = MAXJUL;
	    FrmCustomAlert(HighJulianAlert, maxString, "", "");
	}
	CALLBACK_EPILOGUE
	return newjul;
    }
    else
	CALLBACK_EPILOGUE
	return jul;
}

void DrawLCForm(Boolean redraw) {
    longdate thedate;
    Char baktun[5], katun[5], tun[5], uinal[5], kin[5];
    UInt32 jul = SavedJulian;

    jdntolong(jul, &thedate);

    StrIToA(baktun, thedate.bak   );
    StrIToA(katun, thedate.kat  );
    StrIToA(tun, thedate.tun);
    StrIToA(uinal, thedate.uin  );
    StrIToA(kin, thedate.kin);

    
    SetFieldTextFromStr(FrmGetActiveForm(), BaktunField, baktun, redraw);
    SetFieldTextFromStr(FrmGetActiveForm(), KatunField, katun, redraw);
    SetFieldTextFromStr(FrmGetActiveForm(), TunField, tun, redraw);
    SetFieldTextFromStr(FrmGetActiveForm(), UinalField, uinal, redraw);
    SetFieldTextFromStr(FrmGetActiveForm(), KinField, kin, redraw);



    return;
}
    
void LCIncButtons(EventPtr event){
    
    UInt32 jul = SavedJulian;
    Boolean handled = false;

    switch (event->data.ctlSelect.controlID) {

    case BakIncButton:
	jul = jul + 144000;
	if (jul > MAXJUL)
	    jul = jul - 144000;
	handled = true;
	break;

    case BakDecButton:
	jul = jul - 144000;
	if (jul < JD_CORRELATION)
	    jul = jul + 144000;
	handled = true;
	break;

    case KatIncButton:
	jul = jul + 7200;
	if (jul > MAXJUL)
	    jul = jul - 7200;
	handled = true;
	break;

    case KatDecButton:
	jul = jul - 7200;
	if (jul < JD_CORRELATION)
	    jul = jul + 7200;
	handled = true;
	break;

    case TunIncButton:
	jul = jul + 360;
	if (jul > MAXJUL)
	    jul = jul - 360;
	handled = true;
	break;

    case TunDecButton:
	jul = jul - 360;
	if (jul < JD_CORRELATION)
	    jul = jul + 360;
	handled = true;
	break;

    case UinIncButton:
	jul = jul + 20;
	if (jul > MAXJUL)
	    jul = jul - 20;
	handled = true;
	break;

    case UinDecButton:
	jul = jul - 20;
	if (jul < JD_CORRELATION)
	    jul = jul + 20;
	handled = true;
	break;

    case KinIncButton:
	jul = jul + 1;
	if (jul > MAXJUL)
	    jul = jul - 1;
	handled = true;
	break;

    case KinDecButton:
	jul = jul - 1;
	if (jul < JD_CORRELATION)
	    jul = jul + 1;
	handled = true;
	break;

    }
    SavedJulian = jul;
    DrawLCForm(true);

}

static Boolean LCHandleEvent(EventPtr event){
    Boolean handled = false;
    
    switch (event->eType) {
	
    case ctlRepeatEvent:
	LCIncButtons(event);
	break;
    }	

    return handled;
}


UInt32 SelectLongCount(UInt32 jul) { 
    FormPtr previousForm = FrmGetActiveForm();
    FormPtr frm = FrmInitForm(LongCountForm);
    UInt16 hitButton;
    UInt32 oldjul = SavedJulian;
    UInt32 version;


    FrmSetEventHandler(frm, LCHandleEvent);
    FrmSetActiveForm(frm);
    DrawLCForm(false);
    
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
    }

    hitButton = FrmDoDialog(frm);
    
    if (previousForm)
	FrmSetActiveForm(previousForm);
    
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }


    FrmDeleteForm(frm);
    
    if (hitButton == LCOKButton) {
	return SavedJulian;
    }
    else
	return oldjul;
}

void DrawHaabForm(Boolean redraw) {
    UInt32 jul = SavedJulian;
    Char haabdaystring[4];
    Int32 pushbuttonhit;

    static const char *mnth[] = 
	{ "Pop","Uo","Zip","Zotz","Tzec","Xul","Yaxkin","Mol",
	  "Chen","Yax","Zac","Ceh","Mac","Kankin","Muan","Pax","Kayab",
	  "Cumku","Uayeb" };
    

    if (!(redraw)){
	get_haab_date(jul, &hd1, &hm1);
	hm1--;
	hd2 = hd1;
	hm2 = hm1;
    }

    StrIToA(haabdaystring, hd2);
	
    SetFieldTextFromStr(FrmGetActiveForm(), HaabDayField, 
			haabdaystring, redraw);
    LstSetSelection(GetObjectPtr(NULL, HaabListID), hm2);
    
    if (Next == goNext)
	pushbuttonhit = HaabNextPush;
    else
	pushbuttonhit = HaabPrevPush;

    FrmSetControlGroupSelection(FrmGetActiveForm(), 1, pushbuttonhit);
    CtlSetLabel(
		FrmGetObjectPtr(FrmGetActiveForm() , 
				FrmGetObjectIndex(FrmGetActiveForm() , 
						  HaabTriggerID)) , 
		mnth[hm2]);

    
    return;
}

Boolean HaabPushButtons(EventPtr event) {
    Boolean handled = false;
    
    switch (event->data.ctlSelect.controlID) {
    case HaabPrevPush:
	Next = goPrevious;
	handled = true;
	break;
       
    case HaabNextPush:
	Next = goNext;
	handled = true;
	break;
    }
    return handled;
}
	


Boolean HaabPopup(EventPtr event) {
    hm2 = event->data.popSelect.selection;
    if ((hm2 == 18) && (hd2 > 4))
	hd2 = 4;

    DrawHaabForm(true);
    return true;
}
	

void HaabButtons(EventPtr event) {
    Boolean handled = false;

    switch (event->data.ctlSelect.controlID) {
    case HaabDayIncButton:
	hd2++;
	if (hm2 == 18) {
	    if (hd2 > 4) {
		hd2--;
	    }
	} else if (hd2 > 19)
	    hd2--;
	handled = true;
	break;

    case HaabDayDecButton:
	hd2--;
	if (hd2 < 0)
	    hd2++;
	handled = true;
	break;
    }

    DrawHaabForm(true);
    return;
}



static Boolean HaabHandleEvent(EventPtr event){
    Boolean handled = false;
    
    switch (event->eType) {
	
    case ctlRepeatEvent:
	HaabButtons(event);
	break;
	
    case popSelectEvent:
	handled = HaabPopup(event);
	break;

    case ctlSelectEvent:
	handled = HaabPushButtons(event);
	break;
    }	

    return handled;
}



UInt32 SelectHaab(UInt32 jul) {
    FormPtr previousForm = FrmGetActiveForm();
    FormPtr frm = FrmInitForm(HaabForm);
    UInt16 hitButton;
    UInt32 oldjul = SavedJulian;
    UInt32 newjul;
    Int16 dh1, dh2;
    //    Next = goNext;
    UInt32 version;

    FrmSetEventHandler(frm, HaabHandleEvent);
    FrmSetActiveForm(frm);
    DrawHaabForm(false);

    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
    }
    
    hitButton = FrmDoDialog(frm);
    
    if (previousForm)
	FrmSetActiveForm(previousForm);
    
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }



    FrmDeleteForm(frm);
    
    if (hitButton == HaabOKButton) {
	dh1 = ( (hd1) + 20 * (hm1)) % 365; 
	dh2 = ( (hd2) + 20 * (hm2)) % 365; 
	if ( (dh2 < dh1) && (Next == goNext))
	    dh2 = dh2 + 365;
	if ( (dh2 > dh1) && (Next == goPrevious))
	    dh2 = dh2 - 365; 
	if (dh2 == dh1) 
	    if (Next == goNext)
		dh2 = dh2 + 365;
	    else 
		dh2 = dh2 - 365;
	newjul = oldjul + (dh2 - dh1); // FIXME: add MAX/min check
	if ((newjul > MAXJUL) || (newjul < JD_CORRELATION))
	    newjul = oldjul;
	return newjul;
    }
    else
	return oldjul;
}

void DrawTzolForm(Boolean redraw) {
    UInt32 jul = SavedJulian;
    Char tzoldaystring[4];
    Int32 pushbuttonhit;

    static const char *mnth[] = 
	{"Imix","Ik","Akbal","Kan","Chicchan","Cimi","Manik",
	 "Lamat","Muluc","Oc","Chuen","Eb","Ben","Ix","Men",
	 "Cib","Caban","Eznab","Cauac","Ahau"};

    if (!(redraw)){
	get_tzolkin_date(jul, &td1, &tv1);
	tv1--;
	td2 = td1;
	tv2 = tv1;
    }

    StrIToA(tzoldaystring, td2);
	
    SetFieldTextFromStr(FrmGetActiveForm(), TzolDayField, 
			tzoldaystring, redraw);
    LstSetSelection(GetObjectPtr(NULL, TzolListID), tv2);
    
    if (Next == goNext)
	pushbuttonhit = TzolNextPush;
    else
	pushbuttonhit = TzolPrevPush;

    FrmSetControlGroupSelection(FrmGetActiveForm(), 1, pushbuttonhit);
    CtlSetLabel(
		FrmGetObjectPtr(FrmGetActiveForm() , 
				FrmGetObjectIndex(FrmGetActiveForm() , 
						  TzolTriggerID)) , 
		mnth[tv2]);

    
    return;
}

Boolean TzolPushButtons(EventPtr event) {
    Boolean handled = false;
    
    switch (event->data.ctlSelect.controlID) {
    case TzolPrevPush:
	Next = goPrevious;
	handled = true;
	break;
       
    case TzolNextPush:
	Next = goNext;
	handled = true;
	break;
    }
    return handled;
}
	


Boolean TzolPopup(EventPtr event) {
    tv2 = event->data.popSelect.selection;
 
    DrawTzolForm(true);
    return true;
}
	

void TzolButtons(EventPtr event) {
    Boolean handled = false;

    switch (event->data.ctlSelect.controlID) {
    case TzolDayIncButton:
	td2++;
	if (td2 > 13) {
	    td2--;
	}
	handled = true;
	break;

    case TzolDayDecButton:
	td2--;
	if (td2 < 1)
	    td2++;
	handled = true;
	break;
    }

    DrawTzolForm(true);
    return;
}



static Boolean TzolHandleEvent(EventPtr event){
    Boolean handled = false;
    
    switch (event->eType) {
	
    case ctlRepeatEvent:
	TzolButtons(event);
	break;
	
    case popSelectEvent:
	handled = TzolPopup(event);
	break;

    case ctlSelectEvent:
	handled = TzolPushButtons(event);
	break;
    }	

    return handled;
}



UInt32 SelectTzol(UInt32 jul) {
    FormPtr previousForm = FrmGetActiveForm();
    FormPtr frm = FrmInitForm(TzolForm);
    UInt16 hitButton;
    UInt32 oldjul = SavedJulian;
    UInt32 newjul;
    Int16 dt1, dt2;
    //    Next = goNext;
    UInt32 version;

    FrmSetEventHandler(frm, TzolHandleEvent);
    FrmSetActiveForm(frm);
    DrawTzolForm(false);

    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenModeScaleToFit, rotateModeNone);
    }
    
    hitButton = FrmDoDialog(frm);
    
    if (previousForm)
	FrmSetActiveForm(previousForm);
    
    if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	VgaSetScreenMode(screenMode1To1, rotateModeNone);
    }



    FrmDeleteForm(frm);

    //    if (tv1 == 20)
    //	tv1 = 0;
    // if (tv2 == 20)
    //	tv2 = 0;
    
    if (hitButton == TzolOKButton) {
	dt1 = ((40 * ((td1 - 1) - tv1)) + tv1) % 260;
	dt2 = ((40 * ((td2 - 1) - tv2)) + tv2) % 260;
	if ( (dt2 < dt1) && (Next == goNext))
	    dt2 = dt2 + 260;
	if ( (dt2 > dt1) && (Next == goPrevious))
	    dt2 = dt2 - 260; 
	if (dt2 == dt1) 
	    if (Next == goNext)
		dt2 = dt2 + 260;
	    else
		dt2 = dt2 - 260;
	newjul = oldjul + (dt2 - dt1); // FIXME: add MAX/min check
	if ((newjul > MAXJUL) || (newjul < JD_CORRELATION))
	    newjul = oldjul;
	return newjul;
    }
    else
	return oldjul;
}
    
    


static Boolean MainFormHandleMenuEvent(UInt16 menuID) {
    Boolean handled = false , redraw = false;
    Boolean check = (JD_CORRELATION != standardJDCorr);
    Boolean usebc = USEBC;
    UInt32 jul = SavedJulian;


    switch (menuID) {
    case ToolsNextHaab:
	jul = jul + 365;
	if (jul > MAXJUL)
	    jul = jul - 365;
	handled = true;
	redraw = true;
	break;
	
    case ToolsPrevHaab:
	jul = jul - 365;
	if (jul < JD_CORRELATION)
	    jul = jul + 365;
	handled = true;
	redraw = true;
	break;
	
    case ToolsPrevTzol:
	jul = jul - 260;
	if (jul < JD_CORRELATION)
	    jul = SavedJulian;
	redraw = true;
	handled = true;
	break;

    case ToolsNextTzol:
	jul = jul + 260;
	if (jul > MAXJUL)
	    jul = SavedJulian;
	redraw = true;
	handled = true;
	break;


    case OptionsAbout:
        FrmAlert(AboutBoxAlert);
        handled = true;
        break;
    case OptionsPrefs:
	DoPrefs( &check, &usebc);
	USEBC = usebc;
	JD_CORRELATION=standardJDCorr;
	if (check) {
	    JD_CORRELATION = JD_CORRELATION - 2;
	}
	MAXJUL = JD_CORRELATION + 2879999;
	if (SavedJulian < JD_CORRELATION)
	    SavedJulian = JD_CORRELATION;
	if (SavedJulian > MAXJUL)
	    SavedJulian = MAXJUL;
	redraw = true;
	handled=true;
	break;
    case EditToClip:
	CopyToClipboard(SavedJulian);
	handled = true;
    }
    if (redraw)
	MyDrawData(jul);
    return handled;
}

static Boolean MainFormHandleButtonEvent(EventPtr event) {
    Boolean handled = false;
    UInt16 dd, mm; 
    Int16 yy;
    UInt32 jul, jul2;
    
    date_from_jul(SavedJulian, &dd, &mm, &yy);
    jul = SavedJulian;
    
    switch (event->data.ctlSelect.controlID) {
    case GoHaabButton:
	jul2=SelectHaab(jul);
	jul = jul2;
	handled = true;
	break;

    case GoTzolButton:
	jul2=SelectTzol(jul);
	jul = jul2;
	handled = true;
	break;

    case GoCRButton:
	handled = true;
	break;
    }
    MyDrawData(jul);

    return handled;
}

static Boolean MainFormFieldEvent(EventPtr event) {
    Boolean handled = false;
    UInt16 dd, mm; 
    Int16 yy;
    UInt32 jul, jul2;
    
    date_from_jul(SavedJulian, &dd, &mm, &yy);
    jul = SavedJulian;
    
    switch (event->data.fldEnter.fieldID) {
    case DateField:
	jul2 = MySelectDay(jul);
	jul = jul2;
	handled = true;
	break;

     case JulianField:
	jul2=SelectJulianDay(jul);
	jul = jul2;
	handled=true;
	break;

    case LongCountField:
	jul2=SelectLongCount(jul);
	jul = jul2;
	handled = true;
	break;

    }
    MyDrawData(jul);

    return handled;
}

 


Boolean MainFormForwardOrBack(EventPtr event){ 
    Boolean handled = false;
    UInt32 jul;

    jul = SavedJulian;

    switch (event->data.ctlSelect.controlID) {

    case BackADayButton:
	jul--;
	if (jul < JD_CORRELATION)
	    jul = JD_CORRELATION;
	handled = true;
	break;

    case ForwardADayButton:
	jul++;
	if (jul > MAXJUL)
	    jul = MAXJUL;
	handled = true;
	break;
	
    case PrevTzolButton:
	jul = jul - 260;
	if (jul < JD_CORRELATION)
	    jul = SavedJulian;
	handled = true;
	break;

    case NextTzolButton:
	jul = jul + 260;
	if (jul > MAXJUL)
	    jul = SavedJulian;
	handled = true;
	break;

    case PrevHaabButton:
	jul = jul - 365;
	if (jul < JD_CORRELATION)
	    jul = SavedJulian;
	handled = true;
	break;

    case NextHaabButton:
	jul = jul + 365;
	if (jul > MAXJUL)
	    jul = SavedJulian;
	handled = true;
	break;
	
    case PrevCRButton:
	jul = jul - 18980;
	if (jul < JD_CORRELATION)
	    jul = SavedJulian;
	handled = true;
	break;

    case NextCRButton:
	jul = jul + 18980;
	if (jul > MAXJUL)
	    jul = SavedJulian;
	handled = true;
	break;
	

    }

    MyDrawData(jul);
    

    return handled;
}

Boolean MainFormHardKeys(EventPtr event) {
    Boolean handled = false;
    UInt32 jul;

    jul = SavedJulian;


    switch (event->data.keyDown.chr) {
	
    case pageUpChr:
	jul-- ;
	if (jul < JD_CORRELATION)
	    jul = JD_CORRELATION;
	handled = true;
	break;

    case pageDownChr:
	jul++;
	if (jul > MAXJUL)
	    jul = MAXJUL;
	handled = true;
	break;
    }

    MyDrawData(jul);

    return handled;
}	


static Boolean MyFormHandleEvent(EventPtr event) {
    Boolean handled = false;
    UInt32 version;

    CALLBACK_PROLOGUE

	switch (event->eType) {
	case ctlSelectEvent:
	    handled = MainFormHandleButtonEvent(event);
	    break;
	    

	case ctlRepeatEvent:
	    MainFormForwardOrBack(event);
	    break;


	case ctlExitEvent:
	    handled = true;
	    break;

	case keyDownEvent:
	    handled = MainFormHardKeys(event);
	    break;


	case frmOpenEvent:
	    SizeForm(FrmGetActiveForm());
	    FrmDrawForm(FrmGetActiveForm());
	    if (FirstFlag) {
		DoFirstDraw();
		FirstFlag = false;
	    }
	    handled = true;
	    break;
	    
	case menuEvent:
	    MenuEraseStatus(NULL);
	    handled = MainFormHandleMenuEvent(event->data.menu.itemID);     
	    break;

	case fldEnterEvent:
	    handled = MainFormFieldEvent(event);
	    break;
	    
	case displayExtentChangedEvent:
	    SizeForm(FrmGetActiveForm());
	    MyDrawData(SavedJulian);
	    FrmDrawForm(FrmGetActiveForm());
	    handled = true;
	    break;
		

	}
    
    CALLBACK_EPILOGUE
	return handled;
}

static Boolean ApplicationHandleEvent(EventPtr event){
    FormPtr frm;
    Int16 formId;
    Boolean handled = false;
    UInt32 version;
    UInt32 depthp;
    UInt16 cardNo;

    if (event->eType == frmLoadEvent) {
	formId = event->data.frmLoad.formID;
	frm = FrmInitForm(formId);
	FrmSetActiveForm(frm);

	if (FtrGet(TRGSysFtrID, TRGVgaFtrNum, &version) == 0) {
	    VgaFormModify(frm, vgaFormModify160To240);
	    VgaSetScreenMode(screenMode1To1, rotateModeNone);
	    depthp = 4;
	    WinScreenMode(winScreenModeSet, NULL, NULL, &depthp, NULL);
	    handeradb = DmOpenDatabaseByTypeCreator('strt', 'Maya', dmModeReadOnly);
	    if (handeradb == 0) {
		FrmAlert(NoDBFoundAlert);
		AppLaunchWithCommand( sysFileCLauncher,
				      sysAppLaunchCmdNormalLaunch, NULL );
	    }
    	}
	
	switch (formId) {
	case MainForm:
	    FrmSetEventHandler(frm, MyFormHandleEvent);
	    break;
	}
	handled = true;
    }
    return handled;
}

static void EventLoop(void) {
    EventType event;
    UInt16 error;

	    
    do {
	EvtGetEvent(&event, evtWaitForever);
	if (! SysHandleEvent(&event))
	    if (! MenuHandleEvent(0, &event, &error))
		if (! ApplicationHandleEvent (&event))
		    FrmDispatchEvent(&event);
    } while ( event.eType != appStopEvent);
}

UInt32 PilotMain(UInt16 launchCode, MemPtr cmdPBP, UInt16 launchFlags) {
    Err err = 0;
    if (launchCode == sysAppLaunchCmdNormalLaunch) {
	if ((err = StartApplication()) == 0) {
	    InitDateStuff();
	    EventLoop();
	    StopApplication();
	}
    }
    return err;
}
